@@ -103,7 +103,7 @@ sub _new
         $self = $class->_new_flat( class=>$arg{prop_class}, prop=>$prop[0]{name} );
     }
 
-    if( $self )
+    if( defined $self )
     {
         for( my $i=1; $i<@prop; $i++ )
         {
@@ -158,7 +158,7 @@ sub _new_flat
     }
     else
     {
-        croak "Class '$arg{class}' has no property named '$arg{prop}'";
+        croak "'$arg{prop}' is neither a property of '$arg{class}' nor described in '$class' or parents"
     }
 
     return $self;
